/********************************************************************************
  * @file    PT32x076_adc.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the ADC firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_adc.h"

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_ADC_CR1_MASK           ((u32)0x203F0BFE)   //!< CR1 MASK 
#define Init_ADC_CR2_MASK           ((u32)0x00FF0000)   //!< CR2 MASK
#define ADC_SCHR_SEL    									0x1F
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief ADC Initialization
  * @param ADCx:ADC Peripheral
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_Init(ADC_TypeDef *ADCx, ADC_InitTypeDef *ADC_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Prescaler(ADC_InitStruct->ADC_Prescaler));
	assert_param(IS_ADC_Mode(ADC_InitStruct->ADC_Mode));
	assert_param(IS_ADC_TriggerSource(ADC_InitStruct->ADC_TriggerSource));
	assert_param(IS_ADC_TimerTriggerSource(ADC_InitStruct->ADC_TimerTriggerSource));
	assert_param(IS_ADC_ExternalTriggerSource(ADC_InitStruct->ADC_ExternalTriggerSource));
	assert_param(IS_ADC_Align(ADC_InitStruct->ADC_Align));
	assert_param(IS_ADC_Channel(ADC_InitStruct->ADC_Channel));
	assert_param(IS_ADC_BGVoltage(ADC_InitStruct->ADC_BGVoltage));
	assert_param(IS_ADC_ReferencePositive(ADC_InitStruct->ADC_ReferencePositive));

	tmpreg = ADCx->CR1;
	tmpreg &= (~Init_ADC_CR1_MASK);
	tmpreg |= ((ADC_InitStruct->ADC_Mode) | \
			   (ADC_InitStruct->ADC_TriggerSource) | \
			   (ADC_InitStruct->ADC_TimerTriggerSource) | \
			   (ADC_InitStruct->ADC_ExternalTriggerSource) | \
			   (ADC_InitStruct->ADC_Align) | \
			   (ADC_InitStruct->ADC_Channel) | \
			   (ADC_InitStruct->ADC_BGVoltage) | \
			   (ADC_InitStruct->ADC_ReferencePositive));
	ADCx->CR1 = tmpreg;

	tmpreg = ADCx->CR2;
	tmpreg &= (~Init_ADC_CR2_MASK);
	tmpreg |= ((ADC_InitStruct->ADC_Prescaler) << 16);
	ADCx->CR2 = tmpreg;
}

/**
  * @brief ADC Structure Initialization
  * @param ADC_InitStruct:ADC_InitTypeDef type structure
  * @retval None
  */
void ADC_StructInit(ADC_InitTypeDef *ADC_InitStruct)
{
	ADC_InitStruct->ADC_Prescaler = 2;
	ADC_InitStruct->ADC_Mode = ADC_Mode_Single;
	ADC_InitStruct->ADC_TriggerSource = ADC_TriggerSource_Software;
	ADC_InitStruct->ADC_TimerTriggerSource = ADC_TimerTriggerSource_TIM1ADC;
	ADC_InitStruct->ADC_ExternalTriggerSource = ADC_ExternalTriggerSource_ADCETR;
	ADC_InitStruct->ADC_Align = ADC_Align_Right;
	ADC_InitStruct->ADC_Channel = ADC_Channel_0;
	ADC_InitStruct->ADC_BGVoltage = ADC_BGVoltage_BG1v2;
	ADC_InitStruct->ADC_ReferencePositive = ADC_VoltageReferencePositive_VDDA;
}

/**
  * @brief 	ADC enable command
  * @param 	ADCx:ADC Peripheral
	* @param  NewState: new state of the ADC peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_Cmd(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Set the ADEN bit to Enable the ADC peripheral */
		ADCx->CR1 |= ADC_CR1_EN;
	}
	else
	{
		ADCx->CR1 &= (~ADC_CR1_EN);
	}
}

/**
  * @brief 	ADC Channel Config
  * @param 	ADCx:ADC Peripheral
  * @param 	ADC_Channel:the ADC channel to configure.
  *    @arg   This parameter can be ADC_Channel_x where x can be(0..27)
  * @retval None
  */
void ADC_ChannelConfig(ADC_TypeDef *ADCx, u32 ADC_Channel)
{

	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Channel(ADC_Channel));
	/* Clear the ADC Channel Selection bits */
	ADCx->CR1 &= (u32)~ADC_CR1_CHS;
	/* Configure the ADC Channel */
	ADCx->CR1 |= (u32)ADC_Channel;
}

/**
  * @brief ADC Start Of Conversion
  * @param ADCx:ADC Peripheral
  * @retval None
  */
void ADC_StartOfConversion(ADC_TypeDef *ADCx)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	ADCx->CR1 |= ADC_CR1_SOC;
}

/**
  * @brief ADC Get Conversion Value
  * @param ADCx:ADC Peripheral
  * @retval The conversion value
  */
u16 ADC_GetConversionValue(ADC_TypeDef *ADCx)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));

	/* Return the selected ADC conversion value */
	return (u16) ADCx->DR;
}

/**
  * @brief ADC interrupt config
  * @param ADCx:ADC Peripheral
  * @param ADC_IT: ADC interrupt source
	*          This parameter can be one of the following values:
	* 					@arg ADC_IT_EOC: ADC end of conversion interrupt
	* 					@arg ADC_IT_EOS: ADC end of scan interrupt
 	* @param  NewState: new state of the interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_ITConfig(ADC_TypeDef *ADCx, u32 ADC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_ADC_IT(ADC_IT));

	if(NewState != DISABLE)
	{
		/* Enable the selected ADC interrupts */
		ADCx->CR1 |= ADC_IT;
	}
	else
	{
		/* Disable the selected ADC interrupts */
		ADCx->CR1 &= (~ADC_IT);
	}
}

/**
  * @brief ADC Get Flag Status
  * @param ADCx:ADC Peripheral
  * @param ADC_FLAG: specifies the flag to check.
	*          This parameter can be one of the following values:
	*		 				@arg ADC_FLAG_RDY: ADC ready flag
	*		 				@arg ADC_FLAG_EOC: ADC end of conversion flag
	* 	 				@arg ADC_FLAG_EOS: ADC end of scan flag
  * @retval FlagStatus
  */
FlagStatus ADC_GetFlagStatus(ADC_TypeDef *ADCx, u32 ADC_FLAG)
{
	FlagStatus bitstatus = RESET;

	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_FLAG(ADC_FLAG));

	/* Check the status of the specified ADC flag */
	if((ADCx->SR & ADC_FLAG) != (u32)RESET)
	{
		/* ADC_FLAG is set */
		bitstatus = SET;
	}
	else
	{
		/* ADC_FLAG is reset */
		bitstatus = RESET;
	}

	/* Return the ADC_FLAG status */
	return  bitstatus;
}

/**
  * @brief ADC Average Command
  * @param ADCx:ADC Peripheral
  * @param  NewState: new state of the interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_AverageCmd(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		ADCx->CR1 |= ADC_CR1_AVGE;
	}
	else
	{
		ADCx->CR1 &= (~ADC_CR1_AVGE);
	}
}

/**
  * @brief ADC Average Times Config
  * @param ADCx:ADC Peripheral
  * @param Times: the time of average conversion
  *        @arg IS_ADC_AverageTimes(Times): The value of Times that satisfies the condition
  * @retval None
  */
void ADC_AverageTimesConfig(ADC_TypeDef *ADCx, u32 Times)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_AverageTimes(Times));

	tmpreg = ADCx->CR2;
	tmpreg &= (~ADC_CR2_AVGT);
	tmpreg |= Times;
	ADCx->CR2 = tmpreg;
}

/**
  * @brief ADC Ready Time Config
  * @param ADCx:ADC Peripheral
	* @param ReadyTime:set ready time for the specified ADC
	*          This parameter can be a number between 0x00 and 0xFF.
  * @retval None
  */
void ADC_ReadyTimeConfig(ADC_TypeDef *ADCx, u32 ReadyTime)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ReadyTime(ReadyTime));

	tmpreg = ADCx->CR2;
	tmpreg &= (~ADC_CR2_RDTC);
	tmpreg |= ReadyTime;
	ADCx->CR2 = tmpreg;
}

/**
  * @brief ADC Sample Time Config
  * @param ADCx:ADC Peripheral
  * @param SampleTime:set sample time for the specified ADC
	*          This parameter can be a number between 0x00 and 0x7F.
  * @retval None
  */
void ADC_SampleTimeConfig(ADC_TypeDef *ADCx, u32 SampleTime)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_SampleTime(SampleTime));

	tmpreg = ADCx->CR2;
	tmpreg &= (~ADC_CR2_SMP);
	tmpreg |= ((SampleTime) << 8);
	ADCx->CR2 = tmpreg;
}



/**
  * @brief ADC Scan Command
  * @param ADCx:ADC Peripheral
  * @param  NewState: new state of the scan function.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_ScanCmd(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Set the SCANE bit to Enable the ADC scan */
		ADCx->CR1 |= ADC_CR1_SCANE;
	}
	else
	{
		ADCx->CR1 &= (~ADC_CR1_SCANE);
	}
}

/**
  * @brief ADC Scan Channel Config
  * @param ADCx:ADC Peripheral
  * @param ADC_Channel:the ADC channel to configure.
  *    @arg   This parameter can be ADC_Channel_x where x can be(0..27)
  * @param ScanChannel: ADC scan channel
	*          	This parameter can be a number between 0 and 19
  * @retval None
  */
void ADC_ScanChannelConfig(ADC_TypeDef *ADCx, u32 ScanChannel,u32 ADC_Channel )
{
	u32 tmpreg = 0;
	u32 group, pos;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ScanChannel(ScanChannel));
	assert_param(IS_ADC_Channel(ADC_Channel));

	group = ScanChannel / 6;
	pos = ScanChannel % 6;
	tmpreg = ADCx->SCHR[group];
	tmpreg &= ~(ADC_SCHR_SEL << (pos * 5));
	tmpreg |= ((ADC_Channel >> 16) << (pos * 5));
	ADCx->SCHR[group] = tmpreg;
}

/**
  * @brief ADC Scan Channel Number Config
  * @param ADCx:ADC Peripheral
  * @param ScanNumber: the number of ADC scan channel 
	*          This parameter can be a number between 2 and 20.
  * @retval None
  */
void ADC_ScanChannelNumberConfig(ADC_TypeDef *ADCx, u32 ScanNumber)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ScanChannelNumber(ScanNumber));

	tmpreg = ADCx->SCHR[3];
	tmpreg &= (~ADC_SCHR4_SCNT);
	tmpreg |= ((ScanNumber - 1) << 16);
	ADCx->SCHR[3] = tmpreg;
}

/**
  * @brief ADC DMA Command
  * @param ADCx:ADC Peripheral
  * @param  NewState: new state of the DMA function.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_DMACmd(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Set the DMAE bit to Enable the ADC DMA transition*/
		ADCx->CR1 |= ADC_CR1_DMAE;
	}
	else
	{
		ADCx->CR1 &= (~ADC_CR1_DMAE);
	}
}

/**
  * @brief ADC trigger source config
  * @param ADCx:ADC Peripheral
  * @param SEL: ADC Trigger Source Selection
	*         This parameter can be one of the following values:
	*					@arg ADC_TriggerSource_Software: Software Trigger
	*					@arg ADC_TriggerSource_Timer: Software Trigger
	*					@arg ADC_TriggerSource_ExternalPin: ExternalPin Trigger
  * @retval None
  */
void ADC_TriggerSource(ADC_TypeDef *ADCx, u32 SEL)
{
	/* Check the parameters */
	u32 tmpreg;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_TriggerSource(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= ~((u32)ADC_CR1_TRIGS);
	/* Configure the ADC Clock mode according to ADC_ClockMode */
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}


/**
  * @brief ADC Timer Trigger Source
  * @param ADCx:ADC Peripheral
  * @param SEL: Timer Trigger Source Selection
  *            @arg IS_ADC_TimerTriggerSource(SEL): The value of SEL that satisfies the condition
  * @retval None
  */
void ADC_TimerTriggerSource(ADC_TypeDef *ADCx, u32 SEL)
{
	/* Check the parameters */
	u32 tmpreg;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_TimerTriggerSource(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= ~(ADC_CR1_TIMS);
	/* Configure the ADC Clock mode according to ADC_ClockMode */
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @brief ADC Reference Positive
  * @param ADCx:ADC Peripheral
  * @param SEL: ADC Reference Positive Selection
  *         This parameter can be one of the following values:
	*						@arg ADC_VoltageReferencePositive_VDDA: VDDA
	*						@arg ADC_VoltageReferencePositive_BG2v0: BG2v0
	*						@arg ADC_VoltageReferencePositive_VREF: VREF
  * @retval None
  */
void ADC_ReferencePositive(ADC_TypeDef *ADCx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ReferencePositive(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= (~ADC_CR1_ADVRPS);
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @brief  Configure prescaler for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @param  Division: the prescaler value of the ADC.
  *          This parameter can be a number between 0 and 255.
  * @retval None
  */
void ADC_Prescaler(ADC_TypeDef *ADCx, u32 Division)
{
	/* Check the parameters */
	u32 tmpreg;
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Prescaler(Division));
	tmpreg = ADCx->CR2;
	tmpreg &= ~((u32)ADC_CR2_PSC);
	/* Configure the ADC Clock mode according to ADC_ClockMode */
	tmpreg |= (Division << 16);
	ADCx->CR2 = tmpreg;
}

/**
  * @brief  Configure the mode for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @param  SEL: the mode of the ADC.
  *          This parameter can be one of the following values:
	* 					@arg ADC_Mode_Single: select the single conversion as ADC mode
	* 					@arg ADC_Mode_Continuous: select the continuous conversion as ADC mode
  * @retval None
  */
void ADC_Mode(ADC_TypeDef *ADCx, u32 SEL)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_Mode(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= (~ADC_CR1_MODE);
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @brief  Configure the external trigger source for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @param  SEL: the External Trigger Source of the ADC.
  *          This parameter can be one of the following values:
	* 					@arg ADC_ExternalTriggerSource_ADCETR: select the ADCETR as External Trigger Source
	* 					@arg ADC_ExternalTriggerSource_TIM1CH4: select the TIM1CH4 as External Trigger Source
  * @retval None
  */
void ADC_ExternalTriggerSource(ADC_TypeDef *ADCx, u32 SEL)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_ExternalTriggerSource(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= (~ADC_CR1_EXTS);
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}

/**
  * @brief  Configure the input negative for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @param  SEL: the Input Negative of the ADC.
  *          This parameter can be one of the following values:
	* 					@arg ADC_InputNegative_VSS: select the VSS as Input Negative
	* 					@arg ADC_InputNegative_INN: select the INN as Input Negative
  * @retval None
  */
void ADC_InputNegative(ADC_TypeDef *ADCx, u32 SEL)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_InputNegative(SEL));
	tmpreg = ADCx->CR1;
	tmpreg &= (~ADC_CR1_INNS);
	tmpreg |= SEL;
	ADCx->CR1 = tmpreg;
}


/**
  * @brief 	ADC BG_2v0 config
  * @param 	ADCx:ADC Peripheral
  * @param  NewState: new state of the ADC BG 2v0 mode.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_BG2v0Config(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		ADCx->BGCR |= ADC_BGCR_BGE;
	}
	else
	{
		ADCx->BGCR &= (~ADC_BGCR_BGE);
	}
}

/**
  * @brief 	ADC BG_2v0 output config
  * @param 	ADCx:ADC Peripheral
	* @param  NewState: new state of the ADC output config.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void ADC_BG2v0OutputConfig(ADC_TypeDef *ADCx, FunctionalState NewState)
{
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		ADCx->BGCR |= ADC_BGCR_BGOE;
	}
	else
	{
		ADCx->BGCR &= (~ADC_BGCR_BGOE);
	}
}

/**
  * @brief  Configure the trim value for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @param  Value: the trim value of the ADC.
	* 					This parameter can be a number between 0 and 0x3F.
  * @retval None
  */
void ADC_TrimValueConfig(ADC_TypeDef *ADCx, u32 Value)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));
	assert_param(IS_ADC_BG2v0TrimValue(Value));
	tmpreg = ADCx->BGCR;
	tmpreg &= ~(ADC_BGCR_TRIM | ADC_BGCR_KEY);
	tmpreg |= (Value<<16 | ADC_TrimKey);
	ADCx->BGCR = tmpreg;
}

/**
  * @brief  Get scan conversion value for the specified ADC.
  * @param 	ADCx: ADC Peripheral.
  * @retval The scan conversion value
  */
u16 ADC_GetScanConversionValue(ADC_TypeDef *ADCx)
{
	/* Check the parameters */
	assert_param(IS_ADC_ALL_PERIPH(ADCx));

	/* Return the selected ADC conversion value */
	return (u16) ADCx->DR;
}

/**
  * @}
  */
