/********************************************************************************
  * @file    PT32x076_aes.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the AES firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_aes.h"

/** @defgroup AES
  * @brief AES driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_AES_CR_MASK           ((u32)0x0000001E)   //!< CR MASK 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief Initializes the AESx peripheral according to the specified parameters
  *         in the AES_InitStruct.
  * @param AESx: AES Peripheral.
  * @param AES_InitStruct: pointer to an AES_InitTypeDef structure that contains
  *         the configuration information for the specified AES peripheral.
  * @retval None
  */
void AES_Init(AES_TypeDef *AESx, AES_InitTypeDef *AES_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_OperatingMode(AES_InitStruct->AES_OperatingMode));
	assert_param(IS_AES_EncryptionMethod(AES_InitStruct->AES_EncryptionMethod));
	assert_param(IS_AES_KeySize(AES_InitStruct->AES_KeySize));
	/* Write to tmpreg */
	tmpreg = AES->CR;
	tmpreg &= (u32)~Init_AES_CR_MASK;
	tmpreg  |= (u32)((AES_InitStruct->AES_OperatingMode) | \
									 (AES_InitStruct->AES_EncryptionMethod) | \
									 (AES_InitStruct->AES_KeySize));
	AESx->CR = tmpreg;
}

/**
  * @brief  Fills each AES_InitStruct member with its default value.
  * @param  AES_InitStruct: pointer to an AES_InitTypeDef structure which will be initialized.
  * @retval None
  */
void AES_StructInit(AES_InitTypeDef *AES_InitStruct)
{
	AES_InitStruct->AES_OperatingMode = AES_Mode_Decrypt;
	AES_InitStruct->AES_EncryptionMethod = AES_EncryptionMethodECB;
	AES_InitStruct->AES_KeySize = AES_KeySize128;
}

/**
  * @brief 	Start the computation for the specified AES peripheral.
  * @param 	AESx:AES Peripheral
  * @retval None
  */
void AES_StartComputation(AES_TypeDef *AESx)
{
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	
	AESx->CR |= AES_CR_START;
}

/**
  * @brief Configures operating mode for the specified AES peripheral.
  * @param AESx:AES Peripheral
  * @param Mode:AES_Mode_Decrypt or AES_Mode_Encryption
  * @retval None
  */
void AES_OperatingMode(AES_TypeDef *AESx, u32 Mode)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_OperatingMode(Mode));

	tmpreg = AESx->CR;
	tmpreg &= (~AES_CR_MODE);
	tmpreg |= Mode;
	AESx->CR = tmpreg;
}

/**
  * @brief Configures encryption method for the specified AES peripheral.
  * @param AESx:AES Peripheral
  * @param Mode:AES_EncryptionMethodECB or AES_EncryptionMethodCBC
  * @retval None
  */
void AES_EncryptionMethodConfig(AES_TypeDef *AESx, u32 Method)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_EncryptionMethod(Method));

	tmpreg = AESx->CR;
	tmpreg &= (~AES_CR_EM);
	tmpreg |= Method;
	AESx->CR = tmpreg;
}

/**
  * @brief Configures key size for the specified AES peripheral.
  * @param AESx:AES Peripheral
  * @param KeySize:AES_KeySize128\AES_KeySize192\AES_KeySize256
  * @retval None
  */
void AES_KeySizeConfig(AES_TypeDef *AESx, u32 KeySize)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_KeySize(KeySize));

	tmpreg = AESx->CR;
	tmpreg &= (~AES_CR_KSIZE);
	tmpreg |= KeySize;
	AESx->CR = tmpreg;
}

/**
  * @brief AES KeyExpansionCmd
  * @param AESx:AES Peripheral
  * @retval None
  */
void AES_KeyExpansion(AES_TypeDef *AESx)
{
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));

	AESx->CR |= AES_CR_KEE;
}

/**
  * @brief AES Computation Completed Interrupt
  * @param AESx:AES Peripheral
  * @param NewState:FunctionalState type variable
  * @retval None
  */
void AES_ITConfig(AES_TypeDef *AESx, u32 AES_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_IT(AES_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		AESx->CR |= AES_CR_CCIE;
	}
	else
	{
		AESx->CR &= (~AES_CR_CCIE);
	}
}

/**
  * @brief AES SetKey
  * @param AESx:AES Peripheral
  * @param Keys:Keys Value can be a number between 0x00 and 0xFFFFFFFF.
  * @param KeySize:KeySize Value can be AES_KeySize128\AES_KeySize192\AES_KeySize256.
  * @retval None
  */
void AES_SetKey(AES_TypeDef *AESx,u32 *Keys,u32 KeySize)
{
	u8 i,len;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_KeySize(KeySize));

	if(KeySize == AES_KeySize128)
	{
		len = 4;
	}
	else if(KeySize == AES_KeySize192)
	{
		len = 6;
	}
	else if(KeySize == AES_KeySize256)
	{
		len = 8;
	} 
 
	for(i = 0;i < len; i++)
	{
		AES->KR[i]=Keys[i];
	}
}

/**
  * @brief AES GetKey
  * @param AESx:AES Peripheral
  * @param Keys:Keys Value can be a number between 0x00 and 0xFFFFFFFF.
  * @param KeySize:KeySize Value can be AES_KeySize128\AES_KeySize192\AES_KeySize256.
  * @retval None
  */
void AES_GetKey(AES_TypeDef *AESx,u32 *Keys,u32 KeySize)
{
	u8 i,len;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_KeySize(KeySize));

	if(KeySize == AES_KeySize128)
	{
		len = 4;
	}
	else if(KeySize == AES_KeySize192)
	{
		len = 6;
	}
	else if(KeySize == AES_KeySize256)
	{
		len = 8;
	} 
 
	for(i = 0;i < len; i++)
	{
		Keys[i] = AES->KR[i];
	}
}

/**
  * @brief AES DataInput
  * @param AESx:AES Peripheral
  * @param Din:Din Value can be a number between 0x00 and 0xFFFFFFFF.
  * @retval None
  */
void AES_DataInput(AES_TypeDef *AESx,u32 Din[4])
{
	u8 i;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	for(i=0;i<4;i++)
	{
		AES->DINR[i]=Din[i];
	}
}

/**
  * @brief AES InitializationVectorInput
  * @param AESx:AES Peripheral
  * @param Din:Din Value can be a number between 0x00 and 0xFFFFFFFF.
  * @retval None
  */
void AES_InitializationVectorInput(AES_TypeDef *AESx,u32 IV[4])
{
	u8 i;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	for(i=0;i<4;i++)
	{
		AES->IVR[i]=IV[i];
	}
}

/**
  * @brief AES GetDataOutput
  * @param AESx:AES Peripheral
  * @retval None
  */
void AES_GetDataOutput(AES_TypeDef *AESx,u32 *data)
{
	u8 i;
	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	for(i=0;i<4;i++)
	{
		data[i] = AES->DOUTR[i];
	}
}

/**
  * @brief  Checks whether the specified AES flag is set or not.
  * @param  AESx: select the AES peripheral.
  * @param  AES_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *           @arg AES_FLAG_CCF: computation completed flag
  * @retval The new state of AES_FLAG (SET or RESET).
  */
FlagStatus AES_GetFlagStatus(AES_TypeDef *AESx, u32 AES_FLAG)
{
	FlagStatus bitstatus = RESET;

	/* Check the parameters */
	assert_param(IS_AES_ALL_PERIPH(AESx));
	assert_param(IS_AES_FLAG(AES_FLAG));

	/* Check the status of the specified ADC flag */
	if((AESx->SR & AES_FLAG) != (u32)RESET)
	{
		/* ADC_FLAG is set */
		bitstatus = SET;
	}
	else
	{
		/* ADC_FLAG is reset */
		bitstatus = RESET;
	}

	/* Return the ADC_FLAG status */
	return  bitstatus;
}
/**
  * @}
  */

