/********************************************************************************
  * @file    PT32Sx076_cmp.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the CMP firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_cmp.h"

/** @defgroup CMP
  * @brief CMP driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_CMP_CR1_MASK 		((u32)0x000071FC)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the CMPx peripheral according to the specified parameters
  *         in the CMP_InitStruct.
  * @note   This function is used to configure the global features of the CMP.
  * @param  CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  CMP_InitStruct: pointer to an CMP_InitTypeDef structure that contains
  *         the configuration information for the specified CMP peripheral.
  * @retval None
  */
void CMP_Init(CMP_TypeDef *CMPx, CMP_InitTypeDef *CMP_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_NegativeInput(CMP_InitStruct->CMP_NegativeInput));
	assert_param(IS_CMP_DigitalFilter(CMP_InitStruct->CMP_DigitalFilter));
	assert_param(IS_CMP_PowerMode(CMP_InitStruct->CMP_PowerMode));
	assert_param(IS_CMP_OutputPolarity(CMP_InitStruct->CMP_OutputPolarity));
	assert_param(IS_CMP_PositiveInput(CMP_InitStruct->CMP_PositiveInput));
	assert_param(IS_CMP_InitializationDelayTime(CMP_InitStruct->CMP_InitializationDelayTime));
	/* Write to tmpreg */
	tmpreg = CMPx->CR1;
	tmpreg &= (u32)~Init_CMP_CR1_MASK;
	tmpreg  |= ((u32)((CMP_InitStruct->CMP_NegativeInput) | \
					  (CMP_InitStruct->CMP_DigitalFilter) | \
					  (CMP_InitStruct->CMP_PowerMode)			| \
					  (CMP_InitStruct->CMP_OutputPolarity) | \
					  (CMP_InitStruct->CMP_PositiveInput)));
	/* Write to CMPx CR1 */
	CMPx->CR1 = tmpreg;
	/* Write to CMPx CR2 */
	CMPx->CR2 = ((u32)(CMP_InitStruct->CMP_InitializationDelayTime));
}

/**
  * @brief  Fills each CMP_InitStruct member with its default value.
  * @param  CMP_InitStruct: pointer to an CMP_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void CMP_StructInit(CMP_InitTypeDef *CMP_InitStruct)
{
	CMP_InitStruct->CMP_NegativeInput = CMP_NegativeInput_CMPxN;
	CMP_InitStruct->CMP_DigitalFilter = CMP_DigitalFilter_None;
	CMP_InitStruct->CMP_PowerMode = CMP_PowerMode_HighSpeed;
	CMP_InitStruct->CMP_OutputPolarity = CMP_OutputPolarity_Normal;
	CMP_InitStruct->CMP_PositiveInput = CMP_PositiveInput_CMPxP0;
	CMP_InitStruct->CMP_InitializationDelayTime = 0x00;
}


/**
  * @brief  Enables or disables the specified CMP peripheral.
  * @param  CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  NewState: new state of the CMPx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_Cmd(CMP_TypeDef *CMPx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		CMPx->CR1 |= CMP_CR1_EN;
	}
	else
	{
		CMPx->CR1 &= (~CMP_CR1_EN);
	}
}

/**
  * @brief  Enables or disables the specified CMP interrupts.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  CMP_IT: specifies the OPA interrupt source to be enabled or disabled.
  *          This parameter can be one of the following values:
  *            @arg CMP_IT_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_IT_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_IT_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_IT_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @param  NewState: new state of the specified CMP interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_ITConfig(CMP_TypeDef *CMPx, u32 CMP_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_CMP_IT(CMP_IT));

	if(NewState != DISABLE)
	{
		CMPx->IE |= CMP_IT;
	}
	else
	{
		CMPx->IE &= (~(u32)CMP_IT);
	}
}

/**
  * @brief  Checks whether the specified CMP flag is set or not.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  CMP_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg CMP_FLAG_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_FLAG_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_FLAG_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_FLAG_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval The new state of CMP_FLAG (SET or RESET).
  */
FlagStatus CMP_GetFlagStatus(CMP_TypeDef *CMPx, u32 CMP_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_FLAG(CMP_FLAG));

	/* Check the status of the specified ADC flag */
	if((CMPx->SR & CMP_FLAG) != (u32)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}

	return  bitstatus;
}

/**
  * @brief  Clears the CMPx's pending flags.
  * @param 	CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  CMP_FLAG: specifies the flag to clear.
  *          This parameter can be any combination of the following values:
  *            @arg CMP_FLAG_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_FLAG_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_FLAG_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_FLAG_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval None
  */
void CMP_ClearFlag(CMP_TypeDef *CMPx, u32 CMP_FLAG)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_FLAG(CMP_FLAG));
	CMPx->SR = (u32)CMP_FLAG;
}

/**
  * @brief  Checks whether the specified CMPx's interrupt flag is set or not.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  CMP_IT: specifies the IT to check.
  *          This parameter can be one of the following values:
  *            @arg CMP_IT_COF: Comparator output falling edge trigger interrupt enable control
  *            @arg CMP_IT_COR: Comparator output rising edge trigger interrupt enable control
  *            @arg CMP_IT_COFW: Comparator output falling edge trigger interrupt enable control(Sleep Mode)
  *            @arg CMP_IT_CORW: Comparator output rising edge trigger interrupt enable control(Sleep Mode)
  * @retval The new state of CMP_IT (SET or RESET).
  */
ITStatus CMP_GetITStatus(CMP_TypeDef *CMPx, u32 CMP_IT)
{
	ITStatus bitstatus = RESET;
	u32 enablestatus = 0;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_IT(CMP_IT));
	enablestatus = (u32)(CMPx->IE & CMP_IT);

	if(((u32)(CMPx->SR & CMP_IT) != (u32)RESET) && (enablestatus != (u32)RESET))
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}

	return  bitstatus;
}


/**
  * @brief  Enables or disables output PWM function of the specified CMP peripheral.
  * @param  CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  TIMx_PWMx: select the CMP output PWM.
  *          This parameter can be one of the following values:
  *            @arg IS_CMP_OUTPUT_PWM(TIMx_PWMx): The value of TIMx_PWMx that satisfies the condition
  * @param  NewState: new state of the specified CMP function.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */

void CMP_OutputPWMConfig(CMP_TypeDef *CMPx, u32 TIMx_PWMx)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_OUTPUT_PWM(TIMx_PWMx));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_PWMS);
	tmpreg |= TIMx_PWMx;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Configure the negative intput for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: the negative intput of the CMP.
  *          This parameter can be one of the following values:
  *            @arg CMP_NegativeInput_CMPxN: Select CMPxN as the negative input
  *            @arg CMP_NegativeInput_LDAC: Select LDAC as the negative input
  *            @arg CMP_NegativeInput_BG1V0: Select BG1V0 as the negative input
	*	@retval None
  */
void CMP_NegativeIntputConfig(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_NegativeInput(SEL));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_CNS);
	tmpreg |= SEL;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Configure the digital filter for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: Digital filter level of the CMP.
  *          This parameter can be one of the following values:
  *           @arg IS_CMP_DigitalFilter(SEL): The value of SEL that satisfies the condition   
	*	@retval None
  */
void CMP_DigitalFilterConfig(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_DigitalFilter(SEL));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_DFC);
	tmpreg |= SEL;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Configure the power mode for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: Power mode selection of the CMP.
  *          This parameter can be one of the following values:
  *            @arg CMP_PowerMode_LowPower: Low power consumption mode for optimal power consumption
  *            @arg CMP_PowerMode_HighSpeed: High speed mode for optimal performance
	*	@retval None
  */
void CMP_PowerModeConfig(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_PowerMode(SEL));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_CPMS);
	tmpreg |= SEL;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Configure the output polarity for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: Output polarity selection of the CMP.
  *          This parameter can be one of the following values:
  *            @arg CMP_OutputPolarity_Normal: Normal polarity
  *            @arg CMP_OutputPolarity_Inverted: Reverse polarity
	*	@retval None
  */
void CMP_OutputPolarityConfig(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_OutputPolarity(SEL));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_OPC);
	tmpreg |= SEL;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Configure the positive intput for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: Positive intput selection of the CMP.
  *          This parameter can be one of the following values:
  *           @arg IS_CMP_PositiveInput(SEL): The value of SEL that satisfies the condition
	*	@retval None
  */
void CMP_PositiveIntputConfig(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_PositiveInput(SEL));

	tmpreg = CMPx->CR1;
	tmpreg &= (~CMP_CR1_CPS);
	tmpreg |= SEL;
	CMPx->CR1 = tmpreg;
}

/**
  * @brief  Enables or disables the output synchronization of CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  NewState: new state of the CMPx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_OutputSynchronization(CMP_TypeDef *CMPx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		CMPx->CR1 |= CMP_CR1_SYNC;
	}
	else
	{
		CMPx->CR1 &= (~CMP_CR1_SYNC);
	}
}

/**
  * @brief  Enables or disables the output PWM polarity of CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  NewState: new state of the CMPx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void CMP_OutputPWMPolarity(CMP_TypeDef *CMPx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		CMPx->CR1 |= CMP_CR1_PPC;
	}
	else
	{
		CMPx->CR1 &= (~CMP_CR1_PPC);
	}
}

/**
  * @brief  Configure the initialization delay time for the specified CMP.
  * @param  CMPx: CMPx: where x can be 0 , 1 , 2 or 3 to select the CMP peripheral.
  * @param  SEL: This parameter can be a number between 0 and 0x7FF.
	*	@retval None
  */
void CMP_InitializationDelayTime(CMP_TypeDef *CMPx, u32 SEL)
{
	u32 tmpreg = 0;
	assert_param(IS_CMP_ALL_PERIPH(CMPx));
	assert_param(IS_CMP_InitializationDelayTime(SEL));

	tmpreg = CMPx->CR2;
	tmpreg &= (~CMP_CR2_DELAY);
	tmpreg |= SEL;
	CMPx->CR2 = tmpreg;
}

/**
  * @}
  */
