/********************************************************************************
  * @file    PT32Sx076_dma.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the DMA firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_dma.h"

/** @defgroup DMA
  * @brief DMA driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_DMA_CCR_MASK       ((u32)0x79FE)       //!< CCR MASK 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief Initializes the DMA_Channelx peripheral according to
  *         the specified parameters in the DMA_InitStruct.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  * @param DMA_InitStruct: pointer to a DMA_InitTypeDef
  *         structure that contains the configuration information for
  *         the specified DMA peripheral.
  * @retval None
  */
void DMA_Init(DMA_Channel_TypeDef *DMA_Channelx, DMA_InitTypeDef *DMA_InitStruct)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_DMA_Channel(DMA_Channelx));
	assert_param(IS_DMA_NumberOfData(DMA_InitStruct->DMA_NumberOfData));
	assert_param(IS_DMA_SourceDataSize(DMA_InitStruct->DMA_SourceDataSize));
	assert_param(IS_DMA_DestinationDataSize(DMA_InitStruct->DMA_DestinationDataSize));
	assert_param(IS_DMA_SourceAddressIncrement(DMA_InitStruct->DMA_SourceAddressIncrement));
	assert_param(IS_DMA_DestinationAddressIncrement(DMA_InitStruct->DMA_DestinationAddressIncrement));
	assert_param(IS_DMA_Direction(DMA_InitStruct->DMA_Direction));
	assert_param(IS_DMA_CircularMode(DMA_InitStruct->DMA_CircularMode));
	assert_param(IS_DMA_BURST(DMA_InitStruct->DMA_Burst));

	/* DMA Channel x Configuration */
	DMA_Channelx->CSBAR = DMA_InitStruct->DMA_SourceBaseAddress;
	DMA_Channelx->CDBAR = DMA_InitStruct->DMA_DestinationBaseAddress;
	DMA_Channelx->CNDTR = DMA_InitStruct->DMA_NumberOfData;

	/* Get the DMA_Channelx CCR value */
	tmpreg = DMA_Channelx->CCR;

	/* Clear Configuration bits */
	tmpreg &= ~Init_DMA_CCR_MASK;
	tmpreg |= ((DMA_InitStruct->DMA_SourceDataSize) | \
			   (DMA_InitStruct->DMA_DestinationDataSize) | \
			   (DMA_InitStruct->DMA_SourceAddressIncrement) | \
			   (DMA_InitStruct->DMA_DestinationAddressIncrement) | \
			   (DMA_InitStruct->DMA_Direction) | \
			   (DMA_InitStruct->DMA_CircularMode)) | \
			  (DMA_InitStruct->DMA_Burst);
	DMA_Channelx->CCR = tmpreg;
}

/**
  * @brief Fills each DMA_InitStruct member with its default value.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  *         which will be initialized.
  * @retval None
  */
void DMA_StructInit(DMA_InitTypeDef *DMA_InitStruct)
{
	/* USART_InitStruct members default value */
	DMA_InitStruct->DMA_NumberOfData = 10;
	DMA_InitStruct->DMA_SourceDataSize = DMA_SourceDataSize_Half;
	DMA_InitStruct->DMA_DestinationDataSize = DMA_DestinationDataSize_Half;
	DMA_InitStruct->DMA_SourceAddressIncrement = DMA_SourceAddressIncrement_Enable;
	DMA_InitStruct->DMA_DestinationAddressIncrement = DMA_DestinationAddressIncrement_Enable;
	DMA_InitStruct->DMA_Direction = DMA_Direction_MemoryToMemory;
	DMA_InitStruct->DMA_CircularMode = DMA_CircularMode_Disable;
}

/**
  * @brief Enable or Disable DMA peripheral.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  * @param  NewState: new state of the DMA channel.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void DMA_Cmd(DMA_Channel_TypeDef *DMA_Channelx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_DMA_Channel(DMA_Channelx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the selected DMA_CH peripheral */
		DMA_Channelx->CCR |= DMA_CCR_EN;
	}
	else
	{
		/* Disable the selected DMA_CH peripheral */
		DMA_Channelx->CCR &= (u32)~((u32)DMA_CCR_EN);
	}
}

/**
  * @brief Get DMA's channel current source address.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  * @retval DMA channel current source address
  */
u32 DMA_GetCurrentSourceAddress(DMA_Channel_TypeDef *DMA_Channelx)
{
	/* Check the parameters */
	assert_param(IS_DMA_Channel(DMA_Channelx));
	/*Get DMA_CHx Current Source Address */
	return  DMA_Channelx->CCSAR;
}

/**
  * @brief Get DMA channel current destination address.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  * @retval DMA channel current destination address
  */
u32 DMA_GetCurrentDestinationAddress(DMA_Channel_TypeDef *DMA_Channelx)
{
	/* Check the parameters */
	assert_param(IS_DMA_Channel(DMA_Channelx));
	/*Get DMA_CHx Current Destination Address */
	return  DMA_Channelx->CCDAR;
}

/**
  * @brief Get DMA channel number of data.
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA channel.
  * @retval DMA channel number of data
  */
u16 DMA_GetNumberOfData(DMA_Channel_TypeDef *DMA_Channelx)
{
	/* Check the parameters */
	assert_param(IS_DMA_Channel(DMA_Channelx));

	/*Get DMA_CHx Current Destination Address */
	return  DMA_Channelx->CCNTR;
}

/**
  * @brief Enables or disables the specified DMA interrupts.
  * @param DMAx: select the DMA peripheral.
	* @param DMA_IT: Specify the DMA interrupts sources to be enabled or disabled.
	*          This parameter can be one of the following values:
  *            @arg IS_DMA_IT(x): The value of x that satisfies the condition
  * @param  NewState: new state of the specified DMAx interrupts.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void DMA_ITConfig(DMA_TypeDef *DMAx, u32 DMA_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_DMA_ALL_PERIPH(DMAx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_DMA_IT(DMA_IT));

	if(NewState != DISABLE)
	{
		/* Enable the selected QSPI interrupt */
		DMAx->IER |= DMA_IT;
	}
	else
	{
		/* Disable the selected QSPI interrupt */
		DMAx->IER &= (u32)~DMA_IT;
	}
}

/**
  * @brief Clear DMA interrupt flag
	* @param DMA_FLAG: Specify the DMA interrupts flag to be clear.
	*          This parameter can be one of the following values:
  *            @arg IS_DMA_IT(x): The value of x that satisfies the condition
  * @retval None
  */
void DMA_ClearITFlag(DMA_TypeDef *DMAx, u32 DMA_FLAG)
{
	/* Check the parameters */
	assert_param(IS_DMA_ALL_PERIPH(DMAx));
	assert_param(IS_DMA_IT(DMA_FLAG));

	DMAx->ICR = DMA_FLAG;
}

/**
  * @brief Checks whether the specified DMA flag is set or not.
  * @param DMA_FLAG: Specify the flag to be checked.
	*          This parameter can be one of the following values:
  *            @arg IS_DMA_FLAG(x): The value of x that satisfies the condition
  * @retval The new state of DMA_FLAG (SET or RESET).
  */
FlagStatus DMA_GetFlagStatus(DMA_TypeDef *DMAx, u32 DMA_FLAG)
{
	/* Check the parameters */
	assert_param(IS_DMA_ALL_PERIPH(DMAx));
	assert_param(IS_DMA_FLAG(DMA_FLAG));

	if((DMAx->SR & DMA_FLAG) != 0)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
  * @brief Config DMA Peripheral
  * @param DMA_Channelx:where x can be 0 to 5 to select the DMA peripheral.
	* @param ch: select channel number.
	*         This parameter can be one of the following values:
  *            @arg IS_DMA_ChannelNum(x): The value of x that satisfies the condition
  * @param Peripheral: DAM CH Relevance Peripheral.
  *         This parameter can be one of the following values:
  *            @arg IS_DMA_ChannelxPERIPH(x): The value of x that satisfies the condition
  * @retval None
  */
void DMA_PeripheralConfig(DMA_TypeDef *DMAx, DMA_ChannelNum ch, u32 Peripheral)
{
	/* Check the parameters */
	assert_param(IS_DMA_ALL_PERIPH(DMAx));	
	assert_param(IS_DMA_ChannelNum(ch));
	assert_param(IS_DMA_ChannelxPERIPH(Peripheral));

	DMAx->CHAPCR[ch >> 2] &= ~(DMA_CHAPCR_CH_SEL << ((ch & 0x03) * 8));
	DMAx->CHAPCR[ch >> 2] |= Peripheral << ((ch & 0x03) * 8);
}

/**
  * @}
  */
