/********************************************************************************
  * @file    PT32x076_iwdg.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the IWDG firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_iwdg.h"

/** @defgroup IWDG
  * @brief IWDG driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Enables or disables write access all registers.
  * @param  WDGx:select the WDG peripheral.
  * @param  SEL: new state of write access to all registers.
  *          This parameter can be one of the following values:
  *            @arg WDG_LOCK_UnLock: Enable write access to all registers
  *            @arg WDG_LOCK_Lock: Disable write access to all registers
  * @retval None
  */
void IWDG_LockCmd(IWDG_TypeDef *WDGx, u32 SEL)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_WDG_LockKey(SEL));
	WDGx->LOCK = SEL;
}

/**
  * @brief  Sets WDG Reload value.
  * @param  WDGx:select the WDG peripheral.
  * @param  Reload: specifies the WDG Reload value.
  *          This parameter must be a number between 0x00 and 0xFFFFFFFF.
  * @retval None
  */
void IWDG_SetReload(IWDG_TypeDef *WDGx, u32 Reload)
{
	/* Check the parameters */
	assert_param(IS_WDG_ReloadValue(Reload));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	WDGx->RLR = Reload;
}

/**
  * @brief  Reloads WDG counter
  * @param  WDGx:select the WDG peripheral.
  * @retval None
  */
void IWDG_ReloadCounter(IWDG_TypeDef *WDGx)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	WDGx->KR = 1;
}

/**
  * @brief  Enable or Disable WDGx
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void IWDG_Cmd(IWDG_TypeDef *WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the WDG */
		WDGx->CR |= IWDG_CR_EN;
	}
	else
	{
		/* Disable the WDG */
		WDGx->CR &= ~IWDG_CR_EN;
	}
}

/**
  * @brief  Enables or disables the specified WDG Reset.
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void IWDG_ResetCmd(IWDG_TypeDef *WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the WDG RESET */
		WDGx->CR |= IWDG_CR_RSTE;
	}
	else
	{
		/* Disable the WDG RESET*/
		WDGx->CR &= ~IWDG_CR_RSTE;
	}
}

/**
  * @brief  Enables or disables the specified WDG DebugPending
  * @param  WDGx:select the WDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void IWDG_DBGPendingCmd(IWDG_TypeDef *WDGx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the WDG RESET */
		WDGx->CR |= IWDG_CR_DBGE;
	}
	else
	{
		/* Disable the WDG RESET*/
		WDGx->CR &= ~IWDG_CR_DBGE;
	}
}

/**
  * @brief  Checks whether the specified WDG flag is set or not.
  * @param  WDGx:select the WDG peripheral.
  * @param  WDG_FLAG: specifies the flag to check.
  *          This parameter can be one of the following values:
	*            @arg WDG_FLAG_HungryDog: hungry dog flag
	*            @arg WDG_FLAG_LockStatus: Lock status
  * @retval The new state of WDG_FLAG (SET or RESET).
  */
FlagStatus IWDG_GetFlagStatus(IWDG_TypeDef *WDGx, WDG_FLAG Flag)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_WDG_FLAG(Flag));
	assert_param(IS_WDG_ALL_PERIPH(WDGx));

	if(Flag == WDG_FLAG_HungryDog)
	{
		if((WDGx->SR & IWDG_LOCK_LOCK) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else if(Flag == WDG_FLAG_LockStatus)
	{
		if((WDGx->LOCK & IWDG_LOCK_LOCK) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}

	/* Return the flag status */
	return bitstatus;
}

/**
  * @brief  Configure the clock source for the specified IWDG peripheral.
  * @param 	WDGx: IWDG Peripheral.
  * @param  ClockSource: the clock source of the specified IWDG peripheral.
  *          This parameter can be one of the following values:
	* 					@arg WDG_ClockSource_LSI: select LSI as IWDG peripheral clock source
	* 					@arg WDG_ClockSource_LSE: select LSE as IWDG peripheral clock source
  * @retval None
  */
void IWDG_ClockSourceConfig(IWDG_TypeDef *WDGx, u8 ClockSource)
{
	u8 tempreg;
	/* Check the parameters */
	assert_param(IS_WDG_ALL_PERIPH(WDGx));
	assert_param(IS_WDG_ClockSource(ClockSource));

	tempreg = WDGx->CR;
	tempreg &= ~IWDG_CR_CLKS;
	tempreg |= ClockSource;
	WDGx->CR = tempreg;
}

/**
  * @}
  */
