/********************************************************************************
  * @file    PT32Sx076_lcd.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the LCD firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_lcd.h"

/** @defgroup LCD
  * @brief LCD driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------------------------------------*/
/* Private define ------------------------------------------------------------------------------------------*/
#define Init_LCD_CR_MASK                  ((u32)0x0003B05E)
#define Grayscale_GCR_MASK                ((u32)0x0000000E)
/* Private macro -------------------------------------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------------------------------------*/

/**
  * @brief  Initializes the LCD peripheral according to the specified parameters in the LCD_InitStruct.
  * @note   This function is used to configure the global features of the LCD
  * @param  LCD_InitStruct: pointer to an LCD_InitTypeDef structure that contains
  *         the configuration information for the specified LCD peripheral.
  * @retval None
  */
void LCD_Init(LCD_InitTypeDef *LCD_InitStruct)
{
	u32 tmpreg = 0;

	assert_param(IS_LCD_CRTYPE_SEL(LCD_InitStruct->LCD_DriveVoltageType));
	assert_param(IS_LCD_DUTY_SEL(LCD_InitStruct->LCD_Duty));
	assert_param(IS_LCD_WAVE_SEL(LCD_InitStruct->LCD_Wave));
	assert_param(IS_LCD_CLK_SEL(LCD_InitStruct->LCD_ClkSource));
    assert_param(IS_LCD_PDM_SEL(LCD_InitStruct->LCD_PageDisplayMode));
	assert_param(IS_LCD_RSEL_SEL(LCD_InitStruct->LCD_TotalResistance));
	tmpreg = LCD->CR;
	tmpreg &= ~Init_LCD_CR_MASK;
	tmpreg |=	LCD_InitStruct->LCD_DriveVoltageType |
				LCD_InitStruct->LCD_Duty |
				LCD_InitStruct->LCD_Wave |
				LCD_InitStruct->LCD_ClkSource |
    			LCD_InitStruct->LCD_PageDisplayMode |
				LCD_InitStruct->LCD_TotalResistance;

	LCD->CR = tmpreg;
}


/**
  * @brief  Fills each LCD_InitStruct member with its default value.
  * @note   This function is used to initialize the global features of the LCD
  * @param  LCD_InitStruct: pointer to an LCD_InitTypeDef structure which will be initialized.
  * @retval None
  */
void LCD_StructInit(LCD_InitTypeDef *LCD_InitStruct)
{
	LCD_InitStruct->LCD_DriveVoltageType = LCD_DriveVoltageType_R;		//Rƫѹ
	LCD_InitStruct->LCD_Duty = LCD_Duty_4;		                        // 1/4DUTY,4COM,40SEG
	LCD_InitStruct->LCD_Wave = LCD_Wave_A;			                    //A type
	LCD_InitStruct->LCD_ClkSource = LCD_ClkSource_LSI;		            //ڲ32khzʱ
	LCD_InitStruct->LCD_PageDisplayMode = LCD_PageDisplayMode_Fixed;	//̶ҳʾ
	LCD_InitStruct->LCD_TotalResistance = LCD_TotalResistance_45k;
}

/**
  * @brief  Set the LCD screen control mode.
  * @param  mode: specifies the LCD screen control mode.
  *          This parameter can be any combination of the following values:
  *            @arg LCD_ScreenControlMode_None: Normal display mode
  *            @arg LCD_ScreenControlMode_Blink: Flicker display mode
  *            @arg LCD_ScreenControlMode_ON: Full light display mode
  *            @arg LCD_ScreenControlMode_OFF: All out display mode

  * @retval None
  */
void LCD_ScreenControlModeConfig(u32 mode)
{
     assert_param(IS_LCD_SCM_SEL(mode));
    u32 tmpreg = 0;
    tmpreg = LCD->CR;
	tmpreg &= ~LCD_CR_SCM;
    tmpreg |= mode; 
    LCD->CR = tmpreg ;
}


/**
  * @brief  Clears the LCD all pages buff.
  * @param  None
  * @retval None
  */
void LCD_ClearAllPages(void)
{
	u16 page_offset = 0;
	u16 i = 0, j = 0;

	for(i = 0; i < LCD_MAX_PAGES; i++)
	{
		for(j = 0; j < LCD_PAGE_REGS; j++)
		{
			LCD->LCD_PxLy[page_offset + j] = 0;
		}

		page_offset += LCD_PAGE_REGS;
	}

}

/**
  * @brief		Clears the LCD Page Buff.
  *          		specifies the Page to clear.
  * @param  Page_Data
  *        @arg IS_LCD_Page_SEL(x)  The value of x that satisfies the condition
  * @retval None
  */
void LCD_ClearPage(u32 Page_Data)
{
	u8 j = 0;
	assert_param(IS_LCD_Page_SEL(Page_Data));

	for(j = 0; j < LCD_PAGE_REGS; j++)
	{
		LCD->LCD_PxLy[Page_Data * LCD_PAGE_REGS + j] = 0;
	}
}

/**
  * @brief		Set the LCD Page Buff.
  *          		specifies the Page to set.
  * @param  LCD_PinSource
  *        @arg IS_PinSource_SEL(x)  The value of x that satisfies the condition
  * @param  LCD_COM
  *        @arg IS_LCD_COM_SEL(x)  The value of x that satisfies the condition
  * @param  NewState: new state of the USARTx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_SetDisplayCOM(u32 LCD_PinSource, u32 LCD_COM, FunctionalState NEWState)
{
	u32 i, tmpreg;
	u16  page_offset;

	assert_param(IS_LCD_PinSource_SEL(LCD_PinSource));
	assert_param(IS_LCD_COM_SEL(LCD_COM));
	tmpreg = LCD_COM << (LCD_PinSource & 0x3) * 8;
	page_offset = LCD_PinSource / 4 ;

	if(NEWState == ENABLE)
	{
		for(i = 0; i < LCD_MAX_PAGES; i++)
		{
			LCD->LCD_PxLy[page_offset] |= tmpreg;
			page_offset += LCD_PAGE_REGS;
		}
	}
	else
	{
		for(i = 0; i < LCD_MAX_PAGES; i++)
		{
			LCD->LCD_PxLy[page_offset] &= ~(tmpreg);
			page_offset += LCD_PAGE_REGS;
		}
	}
}


/**
  * @brief  Enables or disables the LCD peripheral.
  * @param  NewState: new state of the LCD peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_Cmd(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->CR |= LCD_CR_EN;
	}
	else
	{
		/* Reset the update bit */
		LCD->CR &= (~LCD_CR_EN);
	}
}


/**
  * @brief  Enables or disables the LCD Data Update
  * @param  NewState: new state of the LCD Data Update
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_UpdataCmd(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->CR |= LCD_CR_DU;
	}
	else
	{
		/* Reset the update bit */
		LCD->CR &= (~LCD_CR_DU);
	}
}


/**
  * @brief  Enables or disables the LCD Gray Adj.
  * @param  NewState: new state of the LCD Gray Adj.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void LCD_GrayscaleCmd(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState == ENABLE)
	{
		/* Set the update bit */
		LCD->GCR |= LCD_GCR_EN;
	}
	else
	{
		/* Reset the update bit */
		LCD->GCR &= (~LCD_GCR_EN);
	}
}

/**
  * @brief  Configures for the LCD Grey Value
  * @param  Gray_Rate: the LCD Grey Value to configure.
  *        @arg IS_LCD_GDR(x)  The value of x that satisfies the condition

  * @retval None
  */
void LCD_Grayscale(u16 Gray_Rate)
{
	u32 tmpreg = 0;
	assert_param(IS_LCD_GDR(Gray_Rate));
	tmpreg = LCD->GCR;
	tmpreg &= ~Grayscale_GCR_MASK;
	tmpreg |= Gray_Rate;
	LCD->GCR = tmpreg;
}

/**
  * @brief  Configures for the LCD Prescaler Value
  * @param  Prescaler: the LCD Prescaler Value to configure.
  * @retval NONE
  */
void LCD_Prescaler(u32 PSC)
{
	assert_param(IS_LCD_PSC_SEL(PSC));
	LCD->PSC = PSC;
}

/**
  * @brief  Configures for the LCD Blink Time Value
  * @param  Blink Time: the LCD Blink Time Value to configure.
  * @retval NONE
  */
void LCD_BlinkTime(u32 BT)
{
	assert_param(IS_LCD_BTCR_SEL(BT));
	LCD->BTCR = BT;
}

/**
  * @brief  Configures for the LCD Blinking Lighting Time Value
  * @param  Blinking Lighting Time: the LCD Blinking Lighting Time Value to configure.
  * @retval NONE
  */
void LCD_BlinkingLightingTime(u32 ON)
{
	assert_param(IS_LCD_BTCR_SEL(ON));
	LCD->BON = ON;
}

/**
  * @brief  Configures for the LCDx Pin Enable
  * @param  LCDx PIN ENABLE: the LCDx PIN to configure.
  * @param  NewState: new state of the USARTx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval NONE
  */
void LCD_PinConfig(u64 LCD_Pin, FunctionalState NewState)
{

	assert_param(IS_LCD_PIN_SEL(LCD_Pin));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		LCD->PCR1 |= (u32)(LCD_Pin & LCD_PCR1_PE);
		LCD->PCR2 |= (u32)((LCD_Pin >> 32) & LCD_PCR2_PE);
	}
	else
	{
		LCD->PCR1 &= ~((u32)(LCD_Pin & LCD_PCR1_PE));
		LCD->PCR2 &= ~((u32)((LCD_Pin >> 32) & LCD_PCR2_PE));
	}
}

/**
  * @brief  Configures for the LCDx Pin Enable
  * @param  LCDx PIN ENABLE: the LCDx PIN to configure.
  * @retval NONE
  */
void LCD_PinDriverConfig(u64 LCD_Pin, u8 driver)
{
	assert_param(IS_LCD_PIN_SEL(LCD_Pin));
	assert_param(IS_LCD_DRIVER_SEL(driver));


	if(driver == LCD_DRIVER_COM)
	{
		LCD->PDCR1 |= (u32)(LCD_Pin & LCD_PDCR1_PDC);
		LCD->PDCR2 |= (u32)((LCD_Pin >> 32) & LCD_PDCR2_PDC);
	}

	else
	{
		LCD->PDCR1 &= ~((u32)(LCD_Pin & LCD_PDCR1_PDC));
		LCD->PDCR2 &= ~((u32)((LCD_Pin >> 32) & LCD_PDCR2_PDC));
	}

}

/**
  * @brief  Configures for the LCDx Page display time
  * @param  Display_Time: the LCDx Display_Time to configure.
  * @retval NONE
  */
void LCD_PageDisplayTime(u16 Display_Time)
{
	assert_param(IS_LCD_PDTR_SEL(Display_Time));

	LCD->PDTR = Display_Time;

}


/**
  * @brief  Configures for the LCDx Scroll display max page
  * @param  Max_Page: the LCDx Scroll display max page to configure.
  * @retval NONE
  */
void LCD_ScrollDisplayPage(u8 Max_Page)
{
	assert_param(IS_LCD_SDPR_SEL(Max_Page));

	LCD->SDPR = Max_Page;

}

/**
  * @brief  Configures for the LCDx Scroll display max page
  * @param  Max_Page: the LCDx Scroll display max page to configure.
  * @retval NONE
  */
void LCD_PageNumberConfig(u8 Page_Number)
{
	assert_param(IS_LCD_PNR_SEL(Page_Number));

	LCD->PNR = Page_Number;

}


/**
  * @brief  Get the status of the register busy flag
  * @retval The new state of BSY
  */
FlagStatus LCD_GetBusyStatus(void)
{
	FlagStatus bitstatus = RESET;

	if((LCD->CR & LCD_CR_BSY) != (u16)RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}

	return bitstatus;
}




/**
  * @}
  */



