/********************************************************************************
  * @file    PT32Sx076_rtc.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the RTC firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_rtc.h"

/** @defgroup RTC
  * @brief RTC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_RTC_CR_MASK	      ((u32)0x0000001E)
#define RTC_UNLOCK_KEY            ((u32)0x1ACCE551)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the RTC peripheral according to the specified parameters
  *         in the RTC_InitStruct.
  * @note   This function is used to configure the global features of the RTC.
  * @param  RTC_InitStruct: pointer to a RTC_InitTypeDef structure that contains
  *         the configuration information for the specified RTC peripheral.
  * @retval None
  */
void RTC_Init(RTC_InitTypeDef *RTC_InitStruct)
{
	u32 tmpreg = 0;
	assert_param(IS_RTC_TF_SET(RTC_InitStruct->RTC_TF));
	assert_param(IS_RTC_CLK(RTC_InitStruct->RTC_ClkSource));
	assert_param(IS_RTC_BEACON(RTC_InitStruct->RTC_BeaconOut));
	assert_param(IS_RTC_TME(RTC_InitStruct->RTC_TrimMode));
	tmpreg = RTC->CR;
	tmpreg &= ~Init_RTC_CR_MASK;
	tmpreg  |=	RTC_InitStruct->RTC_TF |
				RTC_InitStruct->RTC_ClkSource |
				RTC_InitStruct->RTC_BeaconOut |
				RTC_InitStruct->RTC_TrimMode;

	while(RTC->CR & RTC_CR_BSY);
	RTC->CR = tmpreg;
    while(RTC->CR & RTC_CR_BSY);
}

void RTC_StructInit(RTC_InitTypeDef *RTC_InitStruct)
{
	RTC_InitStruct->RTC_TF = RTC_TimeFormat_24;
	RTC_InitStruct->RTC_ClkSource = RTC_ClkSource_LSI;
	RTC_InitStruct->RTC_BeaconOut = RTC_BeaconOut_DISABLE;
	RTC_InitStruct->RTC_TrimMode = RTC_TrimMode_DISABLE;
}


/**
  * @brief  Enables or disables the specified RTC interrupts.
  * @param  RTC_IT: specifies the RTC interrupts sources to be enabled or disabled.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_IT_CHSECE : Half_seconds interrupt
  *     @arg RTC_IT_CSEC: Seconds interrupt
  *     @arg RTC_IT_CMIN: Minutes interrupt
  *     @arg RTC_IT_CHOUR: Hours interrupt
  *     @arg RTC_IT_CDAY: Days interrupt
  *     @arg RTC_IT_CMON: Months interrupt
  *     @arg RTC_IT_ALM: Alarm interrupt
  *     @arg RTC_IT_PEDE: Periodic interrupt
  * @param  NewState: new state of the specified RTC interrupts.
  *   This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RTC_ITConfig(u16 RTC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_RTC_IT(RTC_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		RTC->IE |= RTC_IT;
	}
	else
	{
		RTC->IE &= (u16)~RTC_IT;
	}
}



/**
  * @brief  Clears the RTC's pending flags.
  * @param  RTC_FLAG: specifies the flag to clear.
  *   This parameter can be any combination of the following values:
  *     @arg RTC_FLAG_CHSEC
  *     @arg RTC_FLAG_CSEC
  *     @arg RTC_FLAG_CMIN
  *     @arg RTC_FLAG_CHOUR
  *     @arg RTC_FLAG_CDAY
  *     @arg RTC_FLAG_CMON
  *     @arg RTC_FLAG_ALM
  *     @arg RTC_FLAG_PED
  * @retval None
  */
void RTC_ClearFlag(u8 RTC_FLAG)
{
	/* Check the parameters */
	assert_param(IS_RTC_FLAG(RTC_FLAG));

	/* Clear the corresponding RTC flag */
	RTC->SR2 = RTC_FLAG;
}

/**
  * @brief  Checks whether the specified RTC flag is set or not.
  * @param  RTC_FLAG: specifies the flag to check.
  *   This parameter can be one the following values:
  *     @arg RTC_FLAG_CHSEC
  *     @arg RTC_FLAG_CSEC
  *     @arg RTC_FLAG_CMIN
  *     @arg RTC_FLAG_CHOUR
  *     @arg RTC_FLAG_CDAY
  *     @arg RTC_FLAG_CMON
  *     @arg RTC_FLAG_ALM
  *     @arg RTC_FLAG_PED
  * @retval The new state of RTC_FLAG (SET or RESET).
  */
FlagStatus RTC_GetFlagStatus(u32 RTC_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_RTC_FLAG(RTC_FLAG));

	if((RTC->SR2 & RTC_FLAG) != RESET)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}

	return bitstatus;
}


/**
  * @brief Lock or unlock the calendar register group
  * @param  NewState: new state of the UNLOCK.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval
  */
void RTC_UnlockCmd(FunctionalState NewState)
{
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /*Waiting for the LOCK register to be idle*/
    while(RTC->LOCK & RTC_LOCK_BSY);

	if(NewState == ENABLE)
	{
		RTC->LOCK = RTC_UNLOCK_KEY;
       /*Waiting for unlocking status*/
       while((RTC->LOCK & RTC_LOCK_LOCK));
	}
	else
	{
		RTC->LOCK = 0x0;
       /*Waiting for locking status*/
       while(!(RTC->LOCK & RTC_LOCK_LOCK));
	}
    /*Waiting for the LOCK register to be idle*/
    while(RTC->LOCK & RTC_LOCK_BSY);

}


/**
  * @brief Convert BCD code to decimal
  * @param  Value :BCD code
  * @retval decimal Value
  */
u16 RTC_BCDToDEC(u16 Value)
{
	u16 temp;
	temp = (((Value & 0xF000) >> 12) * 1000 + ((Value & 0xF00) >> 8) * 100 + ((Value & 0xF0) >> 4) * 10 + (Value & 0x0F));
	return temp;
}

/**
  * @brief Convert decimal to  BCD code
  * @param  Value :decimal Value
  * @retval BCD code
  */
u16 RTC_DECToBCD(u16 Value)
{
	u16 temp;
	temp = ((Value / 1000 << 12) + (Value / 100 % 10 << 8) + ((Value / 10 % 10) << 4) + (Value % 10));
	return temp;
}

/**
  * @brief Get Current Date and Time value
  * @param  pDt :struct of date time
  * @retval DEC code
  */
void RTC_GetCalendarTime(RTC_Calendar *CalendarTime)
{
	/*Waiting for RTC to stop counting*/
	while(RTC->SR1 & RTC_SR1_BSY);

	CalendarTime->TimeFormat = ((RTC->CR & RTC_CR_TF) >> 1);	//0: 24-Hour,1: 12-Hour
	CalendarTime->Week = RTC->CWEEK;			//(1~7) 1:monday,...,7 sunday
	CalendarTime->Year = RTC_BCDToDEC(RTC->CYEAR);
	CalendarTime->Month = RTC_BCDToDEC(RTC->CMON);
	CalendarTime->Day = RTC_BCDToDEC(RTC->CDAY);
	CalendarTime->Hour = RTC_BCDToDEC(RTC->CHOUR);
	CalendarTime->Minute = RTC_BCDToDEC(RTC->CMIN);
	CalendarTime->Second = RTC_BCDToDEC(RTC->CSEC);
	CalendarTime->PM = (RTC->CHOUR >> 8);
    
}

/**
  * @brief Set Current Date and Time
  * @param  CalendarTime :struct of date time -- dec format
  * @retval none
  */
void RTC_SetCalendarTime(RTC_Calendar *CalendarTime)
{
	u16 temp1, temp4;
	u8 temp2, temp3, temp5, temp6;
	assert_param(IS_DEC_TimeFormat(CalendarTime->TimeFormat));   //0: 24-Hour,1: 12-Hour
	assert_param(IS_DEC_Week(CalendarTime->Week));
	assert_param(IS_DEC_Year(CalendarTime->Year));
	assert_param(IS_DEC_Month(CalendarTime->Month));
	assert_param(IS_DEC_Day(CalendarTime->Day));
	assert_param(IS_DEC_Hour(CalendarTime->Hour));
	assert_param(IS_DEC_Minute(CalendarTime->Minute));
	assert_param(IS_DEC_Second(CalendarTime->Second));
	assert_param(IS_RTC_AM_PM(CalendarTime->PM));

	/*Waiting for calendar to be idle*/
	while(RTC->CWEEK & RTC_CWEEK_BSY);

	while(RTC->CYEAR & RTC_CYEAR_BSY);

	while(RTC->CMON & RTC_CMON_BSY);

	while(RTC->CDAY & RTC_CDAY_BSY);

	while(RTC->CHOUR & RTC_CHOUR_BSY);

	while(RTC->CMIN & RTC_CMIN_BSY);

	while(RTC->CSEC & RTC_CSEC_BSY);

	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);

	/*Convert to BCD code*/
	temp1 = RTC_DECToBCD(CalendarTime->Year);
	temp2 = RTC_DECToBCD(CalendarTime->Month);
	temp3 = RTC_DECToBCD(CalendarTime->Day);
	temp4 = (RTC_DECToBCD(CalendarTime->Hour)) | ((CalendarTime->PM) << 8);
	temp5 = RTC_DECToBCD(CalendarTime->Minute);
	temp6 = RTC_DECToBCD(CalendarTime->Second);

	RTC_UnlockCmd(ENABLE);
	RTC->CR &= 0xFFFFFFFD;
	RTC->CR |= (CalendarTime->TimeFormat) << 1;   //0: 24-Hour,1: 12-Hour

	RTC->CWEEK = CalendarTime->Week;			//(1~7) 1:monday,...,7 sunday
	RTC->CYEAR = temp1;
	RTC->CMON = temp2;
	RTC->CDAY = temp3;
	RTC->CHOUR = temp4;
	RTC->CMIN = temp5;
	RTC->CSEC = temp6;

	/*Waiting for calendar to be idle*/
	while(RTC->CWEEK & RTC_CWEEK_BSY);

	while(RTC->CYEAR & RTC_CYEAR_BSY);

	while(RTC->CMON & RTC_CMON_BSY);

	while(RTC->CDAY & RTC_CDAY_BSY);

	while(RTC->CHOUR & RTC_CHOUR_BSY);

	while(RTC->CMIN & RTC_CMIN_BSY);

	while(RTC->CSEC & RTC_CSEC_BSY);

	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);
    
}


/**
  * @brief Get Current Alarm Time value
  * @param  AlarmTime :struct of date time
  * @retval DEC code
  */
void RTC_GetAlarmTime(RTC_Alarm *AlarmTime)
{
	AlarmTime->TimeFormat = ((RTC->CR & RTC_CR_TF) >> 1);
	AlarmTime->PM     = ((RTC->AHOUR) >> 8);
	AlarmTime->Hour   = RTC_BCDToDEC(RTC->AHOUR);
	AlarmTime->Minute = RTC_BCDToDEC(RTC->AMIN);
	AlarmTime->Second = RTC_BCDToDEC(RTC->ASEC);
	AlarmTime->Week   = RTC_BCDToDEC(RTC->AWEEK);
}


/**
  * @brief Set Current Date and Time
  * @param  AlarmTime :struct of date time -- dec format
  * @retval none
  */
void RTC_SetAlarmTime(RTC_Alarm *AlarmTime)
{
	u16 temp1;
	u8 temp2, temp3;
	assert_param(IS_DEC_Hour(AlarmTime->Hour));
	assert_param(IS_DEC_Minute(AlarmTime->Minute));
	assert_param(IS_DEC_Second(AlarmTime->Second));
	assert_param(IS_RTC_AWEEK(AlarmTime->Week));
	assert_param(IS_RTC_AM_PM(AlarmTime->PM));

	temp1 = RTC_DECToBCD(AlarmTime->Hour) | ((AlarmTime->PM) << 8);
	temp2 = RTC_DECToBCD(AlarmTime->Minute);
	temp3 = RTC_DECToBCD(AlarmTime->Second);

	RTC_UnlockCmd(ENABLE);
	RTC->AWEEK = AlarmTime->Week;
	RTC->AHOUR = temp1;
	RTC->AMIN  = temp2;
	RTC->ASEC  = temp3;

}

/**
  * @brief Set RTC trim value
  * @param  Value :RTC trim Value
  * @retval none
  */
void RTC_SetTrim(u16 Value)
{
	assert_param(IS_RTC_TrimValue(Value));

	RTC_UnlockCmd(ENABLE);
	RTC->TRIM = Value;

}


/**
  * @brief  Enables or disables the specified RTC peripheral.
  * @param  NewState: new state of the RTC peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RTC_Cmd(FunctionalState NewState)
{
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);
	if(NewState == ENABLE)
	{
		RTC->CR |= RTC_CR_EN;
	}
	else
	{
		RTC->CR &= (~RTC_CR_EN);
	}
	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);
}



void RTC_PeriodInterval(u32 Period)
{
	assert_param(IS_RTC_Period(Period));
	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);
	RTC_UnlockCmd(ENABLE);
	RTC->CR &= ~RTC_CR_PIINR;
	RTC->CR |= (Period << 16);
	/*Waiting for CR register to be idle*/
	while(RTC->CR & RTC_CR_BSY);
}



/**
  * @brief Get Rtc register
  * @param
  * @retval DEC code
  */
u16 RTC_GetTime(u16 RTC_Time)
{
	u16 REG;
	/* Check the parameters */
	assert_param(IS_RTC_TIME(RTC_Time));

	/*Waiting for RTC to stop counting*/
	while(RTC->SR1 & RTC_SR1_BSY);

	switch(RTC_Time)
	{
		case RTC_TIME_SEC:
			REG = RTC->CSEC;
			break;

		case RTC_TIME_MIN:
			REG = RTC->CMIN;
			break;

		case RTC_TIME_HOUR:
			REG = (RTC->CHOUR & 0xFF);
			break;

		case RTC_TIME_DAY:
			REG = RTC->CDAY;
			break;

		case RTC_TIME_WEEK:
			REG = RTC->CWEEK;
			break;

		case RTC_TIME_MON:
			REG = RTC->CMON;
			break;

		case RTC_TIME_YEAR:
			REG = RTC->CYEAR;
			break;

		case RTC_TIME_ASEC:
			REG = RTC->ASEC;
			break;

		case RTC_TIME_AMIN:
			REG = RTC->AMIN;
			break;

		case RTC_TIME_AHOUR:
			REG = RTC->AHOUR;
			break;

		case RTC_TIME_PM:
			REG = ((RTC->CHOUR & RTC_CHOUR_PM) >> 8);
			break;

		default:
			break;
	}

	REG = RTC_BCDToDEC(REG);
	return REG;
}


void RTC_SetTime(u16 RTC_Time, u16 Time)
{
	u16 REG;
	/* Check the parameters */
	assert_param(IS_RTC_TIME(RTC_Time));

	REG = RTC_DECToBCD(Time);
	RTC_UnlockCmd(ENABLE);

	switch(RTC_Time)
	{
		case RTC_TIME_SEC:
			while(RTC->CSEC & RTC_CSEC_BSY);
			RTC->CSEC = REG;
            while(RTC->CSEC & RTC_CSEC_BSY);
			break;

		case RTC_TIME_MIN:
			while(RTC->CMIN & RTC_CMIN_BSY);
			RTC->CMIN = REG;
            while(RTC->CMIN & RTC_CMIN_BSY);
			break;

		case RTC_TIME_HOUR:
			while(RTC->CHOUR & RTC_CHOUR_BSY);
			RTC->CHOUR = REG;
            while(RTC->CHOUR & RTC_CHOUR_BSY);
			break;

		case RTC_TIME_DAY:
			while(RTC->CDAY & RTC_CDAY_BSY);
			RTC->CDAY = REG;
            while(RTC->CDAY & RTC_CDAY_BSY);
			break;

		case RTC_TIME_WEEK:
			while(RTC->CWEEK & RTC_CWEEK_BSY);
			RTC->CWEEK = REG;
            while(RTC->CWEEK & RTC_CWEEK_BSY);
			break;

		case RTC_TIME_MON:
			while(RTC->CMON & RTC_CMON_BSY);
			RTC->CMON = REG;
            while(RTC->CMON & RTC_CMON_BSY);
			break;

		case RTC_TIME_YEAR:
			while(RTC->CYEAR & RTC_CYEAR_BSY);
			RTC->CYEAR = REG;
            while(RTC->CYEAR & RTC_CYEAR_BSY);
			break;

		case RTC_TIME_ASEC:
			RTC->ASEC = REG;
			break;

		case RTC_TIME_AMIN:
			RTC->AMIN = REG;
			break;

		case RTC_TIME_AHOUR:
			RTC->AHOUR = REG;
			break;

		case RTC_TIME_PM:
			RTC->CHOUR = (REG << 8);
			break;

		default:
			break;
	}
}


/**
  * @}
  */

