/********************************************************************************
  * @file    PT32Sx076_spi.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the SPI firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_spi.h"

/** @defgroup SPI
  * @brief SPI driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Init_SPI_CR1_MASK           ((u32)(0xFFFF0030))
#define SlaveOutput_CR2_MASK				((u32)(0xFFFFFFF7))
#define SetDMATC_DCR_Mask						((u32)(0x00000004))
#define TransferDMA_DCR_Mask				((u32)(0x00000002))
#define ReceiveDMA_DCR_Mask					((u32)(0x00000001))
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Initializes the SPIx peripheral according to
  *         the specified parameters in the SPI_InitTypeDef.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  SPI_InitStruct: pointer to a SPI_InitTypeDef
  *         structure that contains the configuration information for
  *         the specified SPI peripheral.
  * @retval None
  */
void SPI_Init(SPI_TypeDef *SPIx, SPI_InitTypeDef *SPI_InitStruct)
{
	u16 tmpreg = 0;
	/* check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	/* Check the SPI parameters */
	assert_param(IS_SPI_MasterSlaveMode(SPI_InitStruct->SPI_MasterSlaveMode));
	assert_param(IS_SPI_DataFrameFormat(SPI_InitStruct->SPI_DataFrameFormat));
	assert_param(IS_SPI_ClockPolarity(SPI_InitStruct->SPI_ClockPolarity));
	assert_param(IS_SPI_ClockPhase(SPI_InitStruct->SPI_ClockPhase));
	assert_param(IS_SPI_CSS(SPI_InitStruct->SPI_CSS));
	assert_param(IS_SPI_SoftwareControlCSS(SPI_InitStruct->SPI_SoftwareControlCSS));
	assert_param(IS_SPI_Prescaler(SPI_InitStruct->SPI_Prescaler));
	assert_param(IS_SPI_BaudRate(SPI_InitStruct->SPI_BaudRate));
	/*---------------------------- SPIx CR1 Configuration ------------------------*/
	/* Get the SPIx CR0 value */
	tmpreg = SPIx->CR1;
	tmpreg &= Init_SPI_CR1_MASK;
	tmpreg |= (u16)((u32)SPI_InitStruct->SPI_DataFrameFormat | SPI_InitStruct->SPI_ClockPolarity |
					SPI_InitStruct->SPI_ClockPhase | ((SPI_InitStruct->SPI_Prescaler) << 8));
	/* Write to SPIx CR1 */
	SPIx->CR1 = tmpreg;
	/*-------------------------SPI Mode Configuration -----------------------*/
	/* Clear MS bits */
	SPIx->CR2 &= (u16)~SPI_CR2_MSM;
	/* Configure SPIx: Mode */
	SPIx->CR2 |= (u16)(SPI_InitStruct->SPI_MasterSlaveMode);
	/*---------------------------- SPIx CPSR Configuration --------------------*/
	/* Write to SPIx Prescaler */
	SPIx->BR = SPI_InitStruct->SPI_BaudRate;

	tmpreg = SPI_InitStruct->SPI_CSS | SPI_InitStruct->SPI_SoftwareControlCSS;
	SPIx->CSS = tmpreg;
}

/**
  * @brief  Fills each SPI_InitStruct member with its default value.
  * @param  SPI_InitStruct: pointer to a SPI_InitTypeDef structure which will be initialized.
  * @retval None
  */
void SPI_StructInit(SPI_InitTypeDef *SPI_InitStruct)
{
	/* Initialize the SPI_MasterSlaveMode member */
	SPI_InitStruct->SPI_MasterSlaveMode = SPI_MasterSlaveMode_Slave;
	/* Initialize the SPI_DataFrameFormat member */
	SPI_InitStruct->SPI_DataFrameFormat = SPI_DataFrameFormat_8b;
	/* Initialize the SPI_ClockPolarity member */
	SPI_InitStruct->SPI_ClockPolarity = SPI_ClockPolarity_Low;
	/* Initialize the SPI_ClockPhase member */
	SPI_InitStruct->SPI_ClockPhase = SPI_ClockPhase_1Edge;
	/* Initialize the SPI_ChipSelectSignal member */
	SPI_InitStruct->SPI_CSS = SPI_CSS_HardwareControl;
	/* Initialize the SPI_SWCS member */
	SPI_InitStruct->SPI_SoftwareControlCSS = SPI_SoftwareControlCSS_High;
	/* Initialize the SPI_PrescalerValue member */
	SPI_InitStruct->SPI_Prescaler = 2;
	/* Initialize the SPI_BaudRate member */
	SPI_InitStruct->SPI_BaudRate = 16;
}

/**
  * @brief  Enables or disables the specified SPI peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  NewState: new state of the SPIx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void SPI_Cmd(SPI_TypeDef *SPIx, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));

	if(NewState != DISABLE)
	{
		/* Enable the selected SPI peripheral */
		SPIx->CR2 |= SPI_CR2_EN;
	}
	else
	{
		/* Disable the selected SPI peripheral */
		SPIx->CR2 &= (u16)~((u16)SPI_CR2_EN);
	}
}

/**
  * @brief  Configures internally by software the CS pin for the selected SPI.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  SWCS_Signal: specifies the selection of SPI CS mode.
  *          This parameter can be one of the following values:
  *            @arg SPI_SoftwareControlCSS_Low: Reset CS pin internally
  *            @arg SPI_SoftwareControlCSS_High: Set CS pin internally
  * @retval None
  */
void SPI_SoftwareControlCSSConfig(SPI_TypeDef *SPIx, u32 SWCS_Signal)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_SoftwareControlCSS(SWCS_Signal));

	if(SWCS_Signal != SPI_SoftwareControlCSS_Low)
	{
		/* Set NSS pin internally by software */
		SPIx->CSS |= SPI_SoftwareControlCSS_High;
	}
	else
	{
		/* Reset NSS pin internally by software */
		SPIx->CSS &= ~(SPI_SoftwareControlCSS_High);
	}
}

/**
  * @brief  Transmits a Data through the SPIx peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  Data: Data to be transmitted.
  * @retval None
  */
void SPI_SendData(SPI_TypeDef *SPIx, u16 Data)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	/* Write in the DR register the data to be sent */
	SPIx->DR = Data;
}

/**
  * @brief  Returns the most recent received data by the SPIx peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @retval The value of the received data.
  */
u16 SPI_ReceiveData(SPI_TypeDef *SPIx)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	/* Return the data in the DR register */
	return SPIx->DR;
}

/**
  * @brief  Enables or disables the specified SPI interrupts.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral
  * @param  SPI_IT: specifies the SPI interrupt source to be enabled or disabled.
  *          This parameter can be one of the following values:
	*            @arg SPI_IT_OVRE: Receive FIFO overflow interrupt
	*            @arg SPI_IT_OTE: Receive FIFO over time interrupt
	*            @arg SPI_IT_RXHE: Receive FIFO half full interrupt
	*            @arg SPI_IT_TXHE: Transmit FIFO more than half interrupt
  * @param  NewState: new state of the specified SPI interrupt.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void SPI_ITConfig(SPI_TypeDef *SPIx, u8 SPI_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_SPI_IT(SPI_IT));

	if(NewState != DISABLE)
	{
		/* Enable the selected SPI interrupt */
		SPIx->IE |= SPI_IT;
	}
	else
	{
		/* Disable the selected SPI interrupt */
		SPIx->IE &= (u16)~SPI_IT;
	}
}

/**
  * @brief  Checks whether the specified SPI flag is set or not.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral
  * @param  SPI_FLAG: specifies the SPI flag to check.
  *          This parameter can be one of the following values:
	*            @arg SPI_FLAG_TXE: Transmit FIFO is empty flag
	*            @arg SPI_FLAG_TNF: Transmit FIFO is not full flag
	*            @arg SPI_FLAG_RXNE: Receive FIFO is not empty flag
	*            @arg SPI_FLAG_RXF: Receive FIFO is full
	*            @arg SPI_FLAG_BSY: Busy flag
	*            @arg SPI_FLAG_OVR: Receive FIFO overflow error flag
	*            @arg SPI_FLAG_OT: Receive FIFO overtime error flag
	*            @arg SPI_FLAG_RXH: Receive FIFO half full flag
	*            @arg SPI_FLAG_TXH: Transmit FIFO more than half flag
  * @retval The new state of SPI_FLAG (SET or RESET).
  */
FlagStatus SPI_GetFlagStatus(SPI_TypeDef *SPIx, u16 SPI_FLAG)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_FLAG(SPI_FLAG));

	/* Check the status of the specified SPI flag */
	if(SPI_FLAG <= SPI_FLAG_BSY)
	{
		if((SPIx->SR1 & (((u16)0x0001) << SPI_FLAG)) != (u16)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else
	{
		if((SPIx->SR2 & (((u16)0x0001) << (SPI_FLAG - 5))) != (u16)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}

	return  bitstatus;
}

/**
  * @brief  Clears the SPIx status flag.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral
  * @param  SPI_FLAG: Specify the status flag to be cleared.
  *         This parameter can be one of the following values:
	*            @arg SPI_FLAG_OVR: Receive FIFO overflow error flag
	*            @arg SPI_FLAG_OT: Receive FIFO overtime error flag
  * @retval None
  */
void SPI_ClearFlag(SPI_TypeDef *SPIx, u16 SPI_FLAG)
{
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_FLAG(SPI_FLAG));

	if(SPI_FLAG == SPI_FLAG_OVR)
	{
		SPIx->IC = SPI_IC_OVR;
	}
	else
	{
		SPIx->IC = SPI_IC_OT;
	}
}

/**
  * @brief  This function reset the SPI receive and transmit FIFOs.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  FIFO: Transmit FIFO or receive FIFO for the SPI.
  *          This parameter can be:
  *            @arg SPI_FIFO_RX: select SPI receive FIFO.
  *            @arg SPI_FIFO_TX: select SPI transmit FIFO.
  * @retval None.
  */
void SPI_FifoReset(SPI_TypeDef *SPIx, u8 SPI_FIFO)
{
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_FIFO(SPI_FIFO));

	if(SPI_FIFO == SPI_FIFO_RX)
	{
		SPIx->RXFR = SPI_RXFR_RXFR;
	}
	else
	{
		SPIx->TXFR = SPI_TXFR_TXFR;
	}
}

/**
  * @brief  Get the number of datas in SPI receive and transmit FIFOs.
  * @param  SPIx:  where x can be 0 or 1 to select the SPI peripheral.
	* @param  FIFO: Transmit FIFO or receive FIFO for the SPI.
  *          This parameter can be:
  *            @arg SPI_FIFO_RX: select SPI receive FIFO.
  *            @arg SPI_FIFO_TX: select SPI transmit FIFO.
	* @retval num: the number of datas in SPI FIFOs.
  */
u8 SPI_GetFifoSize(SPI_TypeDef *SPIx, u8 SPI_FIFO)
{
	uint8_t num = 0;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_FIFO(SPI_FIFO));

	if(SPI_FIFO == SPI_FIFO_RX)
	{
		num = (SPIx->SR1 & 0xF00) >> 8;
	}
	else
	{
		num = (SPIx->SR1 & 0xF000) >> 12;
	}

	return num;
}

/**
  * @brief  Enables or disables DMA transfer request of the specified SPI peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  NewState: new state of the SPIx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void SPI_TransferDMACmd(SPI_TypeDef *SPIx, FunctionalState NewState)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	tmpreg = SPIx->DCR;
	tmpreg &= ~(TransferDMA_DCR_Mask);
	if(NewState != DISABLE)
	{
		/* Enable the SPI slave output */
		tmpreg |= SPI_DCR_TXDMAE;
		
	}
	else
	{
		/* Disable the SPI slave output */
		tmpreg &= ~(SPI_DCR_TXDMAE);
	}
	SPIx->DCR = tmpreg;
}

/**
  * @brief  Enables or disables DMA receive request of the specified SPI peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  NewState: new state of the SPIx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void SPI_ReceiveDMACmd(SPI_TypeDef *SPIx, FunctionalState NewState)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	tmpreg = SPIx->DCR;
	tmpreg &= ~(ReceiveDMA_DCR_Mask);
	if(NewState != DISABLE)
	{
		/* Enable the SPI slave output */
		tmpreg |= SPI_DCR_RXDMAE;
	}
	else
	{
		/* Disable the SPI slave output */
		tmpreg &= ~(SPI_DCR_RXDMAE);
	}
	SPIx->DCR = tmpreg;
}

/**
  * @brief  Set DMA trigger condition of the specified SPI peripheral.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  DMA_TriggerControl: specifies the selection of DMA trigger condition.
  *          This parameter can be one of the following values:
  *            @arg SPI_DMA_TriggerControl_1: Set DMA trigger condition as 1 frame
  *            @arg SPI_DMA_TriggerControl_4: Set DMA trigger condition as 4 frame
  * @retval None
  */
void SPI_SetDMATC(SPI_TypeDef *SPIx, u32 DMA_TriggerControl)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_SPI_DMATrigger(DMA_TriggerControl));
	tmpreg = SPIx->DCR;
	tmpreg &= ~(SetDMATC_DCR_Mask);
	if(DMA_TriggerControl != SPI_DMA_TriggerControl_4)
	{
		/* Enable the SPI slave output */
		tmpreg |= SPI_DMA_TriggerControl_1;
	}
	else
	{
		/* Disable the SPI slave output */
		tmpreg |= SPI_DMA_TriggerControl_4;
	}
	SPIx->DCR = tmpreg;
}

/**
  * @brief  Enables or disables the specified SPI peripheral salve output in slave mode.
  * @param  SPIx: where x can be 0 or 1 to select the SPI peripheral.
  * @param  NewState: new state of the SPIx peripheral.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void SPI_SlaveOutputConfig(SPI_TypeDef *SPIx, FunctionalState NewState)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_SPI_ALL_PERIPH(SPIx));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	tmpreg = SPIx->CR2;
	tmpreg &= SlaveOutput_CR2_MASK;

	if(NewState != DISABLE)
	{
		/* Enable the SPI slave output */
		tmpreg &= ~(SPI_CR2_SOD);
	}
	else
	{
		/* Disable the SPI slave output */
		tmpreg |= SPI_CR2_SOD;
	}
	SPIx->CR2 = tmpreg;
}

/**
  * @}
  */
