/********************************************************************************
  * @file    PT32Sx076_wwdg.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2023/7/15
  * @brief   This file contains all the functions prototypes for the WWDG firmware
  *          library
  * @attention
  *
  * ǰĹָ̼, ĿͻṩйƷıϢ,Խʡǵʱ䡣
  * ̼/ͻʹаıϢκ, * Pai-IC κֱӣӻ̷𺦸
  *
  * (C) Ȩ Pai-IC Microelectronics
  *******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x076_wwdg.h"

/** @defgroup WDG
  * @brief WDG driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define CFR_WDGTB_Mask    ((u32)0x00007000)
#define CFR_W_Mask        ((u32)0x0000007F)
#define BIT_Mask      ((u32)0x0000007F)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Enable or Disable WWDG
  * @param  WDGx:select the WWDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WWDG_Cmd(WWDG_TypeDef* WDGx, FunctionalState NewState)
{
      /* Check the parameters */
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));	
    assert_param(IS_FUNCTIONAL_STATE(NewState));	
  if (NewState != DISABLE)
  {
    /* Enable the WWDG CLOCK */
		WDGx->CR2 = WWDG_CR2_EN | WWDG_CR2_KEYVALUE;
  }
  else
  {
    /* Disable the WWDG CLOCK*/
		WDGx->CR2 = (~WWDG_CR2_EN) | WWDG_CR2_KEYVALUE;
  }
    
}

/**
  * @brief  Enables the specified WWDG Reset.
  * @param  WWDGx:select the WWDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WWDG_ResetEnable(WWDG_TypeDef* WDGx)
{
  /* Check the parameters */
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));	
    WDGx->CR1 |= WWDG_CR1_RSTE;

}

/**
  * @brief  Sets the WWDG Prescaler.
  * @param  WWDG_Prescaler: specifies the WWDG Prescaler.
  *   This parameter can be one of the following values:
  *     @arg WWDG_Prescaler_1: WWDG counter clock = (PCLK1/2048)/1
  *     @arg WWDG_Prescaler_2: WWDG counter clock = (PCLK1/2048)/2
  *     @arg WWDG_Prescaler_4: WWDG counter clock = (PCLK1/2048)/4
  *     @arg WWDG_Prescaler_8: WWDG counter clock = (PCLK1/2048)/8
  *     @arg WWDG_Prescaler_16: WWDG counter clock = (PCLK1/2048)/16
  *     @arg WWDG_Prescaler_32: WWDG counter clock = (PCLK1/2048)/32
  *     @arg WWDG_Prescaler_64: WWDG counter clock = (PCLK1/2048)/64
  *     @arg WWDG_Prescaler_128: WWDG counter clock = (PCLK1/2048)/128
  * @retval None
  */
void WWDG_SetPrescaler(u32 WWDG_Prescaler)
{
  u32 tmpreg = 0;
  /* Check the parameters */
  assert_param(IS_WWDG_Prescaler(WWDG_Prescaler) ) ;
  /* Clear WDGTB[1:0] bits */
  tmpreg = WWDG->CFGR & ~CFR_WDGTB_Mask;
  /* Set WDGTB[1:0] bits according to WWDG_Prescaler value */
  tmpreg |= WWDG_Prescaler;
  /* Store the new value */
  WWDG->CFGR = tmpreg;
}

/**
  * @brief  Enables or disables the specified WWDG DebugPending
  * @param  WDGx:select the WWDG peripheral.
  * @param  NewState: new state of the WDGx peripheral.
	*				   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void WWDG_DBGPendingCmd(WWDG_TypeDef* WDGx, FunctionalState NewState)
{
  /* Check the parameters */
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));	
  assert_param(IS_FUNCTIONAL_STATE(NewState));	
  if (NewState != DISABLE)
  {
    /* Enable the WWDG RESET */
    WDGx->CFGR |= WWDG_CFGR_DBGE;
  }
  else
  {
    /* Disable the WWDG RESET*/
    WDGx->CFGR &= ~WWDG_CFGR_DBGE;
  }
}

/**
  * @brief  Sets the WWDG window value.
  * @param  WindowValue: specifies the window value to be compared to the downcounter.
  *   This parameter value must be lower than 0x80.
  * @retval None
  */
void WWDG_SetWindowValue(u32 WindowValue)
{
  u32 tmpreg = 0;

  /* Check the parameters */
  assert_param(IS_WWDG_WINDOW_VALUE(WindowValue));
  /* Clear W[6:0] bits */
  tmpreg = WWDG->CFGR & ~CFR_W_Mask;
  /* Set W[6:0] bits according to WindowValue value */
  tmpreg |= WindowValue;
  /* Store the new value */
  WWDG->CFGR = tmpreg;
}

/**
  * @brief  Enables the WWDG Early Wakeup interrupt(EWI).
  * @param  None
  * @retval None
  */
void WWDG_ITConfigEnable(WWDG_TypeDef* WDGx,u16 WWDG_IT )
{
    /* Check the parameters */
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));		
  	assert_param(IS_WWDG_IT(WWDG_IT));
	if((WWDG_IT & WWDG_IT_EWIE)==WWDG_IT_EWIE)
	{
		/* Enable the WDG */
		WDGx->CFGR |= WWDG_IT_EWIE;
	}
}

/**
  * @brief  Sets the WWDG counter value.
  * @param  Counter: specifies the watchdog counter value.
  *   This parameter must be a number between 0x40 and 0x7F.
  * @retval None
  */
void WWDG_SetCounter(u32 Counter)
{
  /* Check the parameters */
  assert_param(IS_WWDG_COUNTER(Counter));
  /* Write to T[6:0] bits to configure the counter value, no need to do
     a read-modify-write; writing a 0 to WDGA bit does nothing */
  WWDG->CR1 = Counter & BIT_Mask;
}

/**
  * @brief  Checks whether the Early Wakeup interrupt flag is set or not.
  * @param  None
  * @retval The new state of the Early Wakeup interrupt flag (SET or RESET)
  */
FlagStatus WWDG_GetFlagStatus(WWDG_TypeDef* WDGx, WWDG_FLAG Flag)
{
	FlagStatus bitstatus = RESET;
	/* Check the parameters */
	assert_param(IS_WWDG_FLAG(Flag));
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));

  if(Flag == WWDG_FLAG_EWIF)
	{
		if((WDGx->SR & WWDG_SR_EWIF) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}

	/* Return the flag status */
	return bitstatus;
}
/**
  * @brief  Clears Early Wakeup interrupt flag.
  * @param  None
  * @retval None
  */
void WWDG_ClearFlag(WWDG_TypeDef* WDGx, u8 WWDG_FLAG)
{
	assert_param(IS_WWDG_ALL_PERIPH(WDGx));
	assert_param(IS_WWDG_FLAG(WWDG_FLAG));

  if(WWDG_FLAG == WWDG_FLAG_EWIF)
	{
		WDGx->SR = WWDG_SR_EWIF;
	}

}
/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */
